@@ -10,21 +10,33 @@
  */
 
 RETSIGTYPE
+#ifdef _HAVE_PROTOS
+fptrap (int sig)
+#else
 fptrap (sig)
      int sig;
+#endif
 {
   rterror ("arithmetic exception");
 }
 
 void
+#ifdef _HAVE_PROTOS
+setflt (void)
+#else
 setflt ()
+#endif
 {
   if (signal (SIGFPE, SIG_IGN) != SIG_IGN)
     signal (SIGFPE, fptrap);
 }
 
 void
+#ifdef _HAVE_PROTOS
+resetflt (void)
+#else
 resetflt ()
+#endif
 {
   if (signal (SIGFPE, SIG_IGN) != SIG_IGN)
     signal (SIGFPE, SIG_DFL);
